<?php
/**
 * @file
 * Defines the ControllerMenuItemsNormalizer and utility classes.
 */

/**
 * Normalizes menu item definition to the notation that Drupal uses.
 */
class ControllerMenuItemNormalizer implements ControllerMenuItemNormalizerInterface {
  function accept($path, array $definition) {
    return TRUE;
  }

  /**
   * Normalizes menu item definition.
   *
   * @return array
   */
  function normalize($path, array $definition) {
    $this->checkDefinition($definition);

    // 1. Add some standard keys if they missing.
    $definition += $this->getDefaultItems();

    $definition['page callback'] = CONTROLLER_MENU_HANDLER;

    if (!isset($definition['page arguments'])) {
      $definition['page arguments'] = array();
    }

    $controller_class_name = $definition['controller'];
    $action_name = $definition['action'];
    unset($definition['action'], $definition['controller']);

    $definition['page arguments'] = array_merge(
      array(
        $controller_class_name
        . CONTROLLER_MENU_SEPARATOR
        . $action_name
        . CONTROLLER_MENU_SEPARATOR
        . $definition['module'],
      ),
      $definition['page arguments']
    );

    return array($path => $definition);
  }

  /**
   * Returns menu type.
   */
  protected function getType() {
    return MENU_CALLBACK;
  }

  /**
   * Checks menu item definition. Should throw an exception
   * if something is invalid in it.
   */
  protected function checkDefinition(array $definition) {
    $this->checkRequiredKeys($this->getRequiredKeys(), $definition);
  }

  protected function checkRequiredKeys(array $keys, array $definition) {
    foreach ($keys as $key) {
      if (!isset($definition[$key])) {
        throw new RuntimeException("Missing required key '$key' in the menu item definition: " . print_r($definition, 1));
      }
    }
  }

  protected function getRequiredKeys() {
    return array('module', 'controller', 'action');
  }

  protected function getDefaultItems() {
    return array(
      'type' => $this->getType(),
      'weight' => 0,
      'title' => '',
    );
  }
}

/**
 * Represents extended version of the ControllerMenuItemNormalizer that
 * can handle the 'default' key.
 */
class ControllerDefaultMenuItemNormalizer extends ControllerMenuItemNormalizer {
  public function accept($path, array $definition) {
    return isset($definition['default']);
  }

  /**
   * Normalizes menu item definition.
   *
   * @return array
   */
  public function normalize($path, array $definition) {
    $definition += array(
      'type' => MENU_NORMAL_ITEM,
    );
    $parent_definition = parent::normalize($path, $definition);
    $parent_path = $path;

    // Get title and path from the parent item if neccessary.
    $default_item = $parent_definition[$parent_path]['default'];
    $path = $parent_path . '/' . trim(key($default_item), '/');
    $title = reset($default_item);
    unset($parent_definition[$parent_path]['default']);

    $definition = array(
      $path => array_merge(
        $this->getDefaultItems(),
        array(
          'title' => $title,
          'type' => MENU_DEFAULT_LOCAL_TASK,
          'file' => $parent_definition[$parent_path]['file'],
          'module' => $parent_definition[$parent_path]['module'],
        )
      ),
    );

    return array_merge($parent_definition, $definition);
  }

  /**
   * Checks menu item definition. Should throw an exception
   * if something is invalid in it.
   */
  protected function checkDefinition(array $definition) {
    parent::checkDefinition($definition);
    if (!is_array($definition['default'])) {
      throw new RuntimeException("Invalid format the menu item: " . print_r($definition, 1));
    }
  }
}

class ControllerStubMenuItemNormalizer implements ControllerMenuItemNormalizerInterface {
  public function accept($path, array $definition) {
    return TRUE;
  }

  public function normalize($path, array $definition) {
    return array($path => $definition);
  }
}
